# -*- coding: ISO-8859-15 -*-
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
# Author: Lennart Regebro <regebro@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: calendartool.py 27880 2005-10-03 17:09:53Z lregebro $
"""
  calendartool.py
"""
from zope.interface import implements

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from AccessControl.Permissions import manage_users 
from AccessControl.PermissionRole import rolesForPermissionOn
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from Persistence import PersistentMapping

from Products.ZCatalog.ZCatalog import ZCatalog
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore.utils import _getAuthenticatedUser, _checkPermission
from Products.CMFCalendar.CalendarTool import CalendarTool as CMFCalendarTool
# Please let this backwards compatibility stay until CPS 3.4.0
from Products.CMFCore.CMFCorePermissions import View, setDefaultRoles
from Products.CMFCore.CMFCorePermissions import AddPortalContent

import Products.CalCore # Force it's loading and pythonpath patching
from calcore.interfaces import IAttendeeSource
from Products.CalZope.zopecal import StorageManager, Calendar, Attendee

from zope.i18nmessageid import MessageIDFactory
_ = MessageIDFactory("Default")

# id for default personal calendars
CALENDAR_ID = 'calendar'

EVENT_LIST = ['invite', 'status_change', 'deleted', 'modify']

class CalendarTool(CMFCalendarTool, Folder):

    meta_type = 'CPS Calendar Tool'
    id = 'portal_calendar'
    # Backwards compatibility with CMFCalendarTool:
    meta_types = ['Event'] 
                   
    def manage_afterAdd(self, item, container):
        ob = CpsStorageManager()
        self._setObject(ob.getId(), ob)
        ob = CpsAttendeeSource()
        self._setObject(ob.getId(), ob)
                
    def _getAttendeeSource(self):
        return self.attendee_source

    def _getCpsCalendarTool(self):
        return getToolByName(self, 'portal_cpscalendar')


class CpsStorageManager(StorageManager):
    """A storage manager that does notifications"""
    implements(IAttendeeSource)

    meta_type = "CPS Storage Manager"
    id = 'storage_manager'

    def __init__(self):
        CpsStorageManager.inheritedAttribute('__init__')(self, self.id, '')

    def _getCalendarTool(self):
        return self.aq_parent

    def _getCpsCalendarTool(self):
        return getToolByName(self, 'portal_cpscalendar')

    def notifyOnStatusChange(self, attendee, event, old_status, new_status):
        self._storage.reindexEvent(event.unique_id, event)
        # This part determines what notification event should be sent, and
        # to whom. If we end up needing more flexibility here, this should
        # be moved to a configuration page in the tool.
        event_tool = getToolByName(self, 'portal_eventservice')
        ctool = self._getCpsCalendarTool()
        asrc = self._getCalendarTool().attendee_source
        organizer = asrc.getAttendee(event.getOrganizerId())
        if organizer.getAttendeeId() == attendee.getAttendeeId():
            notify_attendees = [attendee]
        else:
            notify_attendees = [attendee, organizer]

        # By default, notify the attendee of everything.
        if old_status is None and new_status == 'NEEDS-ACTION':
            event_type = 'calendar_event_invite'
        elif new_status is None:
            event_type = 'calendar_event_deleted'
        else:
            event_type = 'calendar_event_status_change'
        
        infos = {'old_status': _(old_status).encode('ISO-8859-15'), 
                 'new_status': _(new_status).encode('ISO-8859-15'),
                 'event_begins': str(event.dtstart),
                 'event_ends': str(event.dtstart + event.duration),
                 'event_title': event.title.encode('ISO-8859-15'),
                }
        
        current_attendee_id = asrc.getCurrentUserAttendee().getAttendeeId()
        for attendee in notify_attendees:
            # Don't send a notification to yourself.
            if attendee.getAttendeeId() == current_attendee_id:
                continue
            homecal = ctool.getHomeCalendarObject(attendee.getUserId())
            if homecal is None:
                continue
            # Generate the correct event_url:
            url = '/'.join((homecal.absolute_url(), 'event', event.unique_id))
            infos['event_url'] = url

            event_tool.notify(event_type, homecal, infos)
     
    def notifyOnEventModification(self, event):
        """Notify on event modification. 
        
        kw holds a dict of attributes with the old values.
        """
        self._storage.reindexEvent(event.unique_id, event)
        
        infos = {'event_begins': str(event.dtstart),
                 'event_ends': str(event.dtstart + event.duration),
                 'event_title': event.title.encode('ISO-8859-15'),
                }
        notification = 'calendar_event_modify'

        event_tool = getToolByName(self, 'portal_eventservice')
        ctool = self._getCpsCalendarTool()
        asrc = self._getCalendarTool().attendee_source
        current_attendee_id = asrc.getCurrentUserAttendee().getAttendeeId()
        attendees = [asrc.getAttendee(id) for id in event.getAttendeeIds()]

        for attendee in attendees:
            # Don't send a notification to yourself.
            if attendee.getAttendeeId() == current_attendee_id:
                continue
            # Only notify people that are coming.
            if event.getParticipationStatus(attendee) not in (
                'ACCEPTED', 'TENTATIVE'):
                continue
            homecal = ctool.getHomeCalendarObject(attendee.getUserId())
            # Can't notify people without home calendar
            if homecal is None:
                continue
            # Generate the correct event_url:
            url = '/'.join((homecal.absolute_url(), 'event', event.unique_id))
            infos['event_url'] = url
            event_tool.notify(notification, homecal, infos)

class CpsAttendeeSource(UniqueObject, SimpleItem):
    
    implements(IAttendeeSource)

    meta_type = "CPS Attendee Source"
    id = 'attendee_source'

    def _getMembersDirectory(self):
        return getToolByName(self, 'portal_directories').members

    def getAttendee(self, attendee_id):
        # Look up member
        prefix, id = attendee_id.split('!', 1)
        
        if prefix == 'INDIVIDUAL':
            members = self._getMembersDirectory()
            try:
                member = members._getEntry(id)
                # Use a get since the directory may be misconfigured:
                title = member.get(members.title_field, '')
                if not title: 
                    title = 'User %s' % id
            except KeyError:
                title = 'Non-existing attendee (%s)' % id
            type = prefix
        elif prefix == 'CALENDAR':
            calendar = self.unrestrictedTraverse(id, None)
            if calendar is None: # Most likely a calendar that has been deleted
                type = 'UNKNOWN'
                title = 'Non-existing attendee (%s)' % id
            else:
                # During upgrade, you sometimes find old calendars.
                # These do not have an _attendee_type. Default to
                # UNKNOWN. 
                type = getattr(calendar, '_attendee_type', 'UNKNOWN')
                title = calendar.title_or_id()
        else:
            raise 'Unknown attendee prefix: %s' % prefix
        attendee = CpsAttendee(id, title, type)
        homecal = self.getMainCalendarForAttendeeId(attendee_id)
        if homecal is None:
            return attendee.__of__(self)
        # This is to acquire the security settings from the calendar:
        return attendee.__of__(homecal)
        

    def findByName(self, query_str, attendee_type=None):
        """Search for attendees"""
        if query_str == '': # Empty string means no match
            return []

        if query_str == '*': # Asterix means "everything"
            query_str  = ''
            
        catalog = getToolByName(self, 'portal_catalog')
        search = ZCatalog.searchResults(catalog, meta_type='CPS Calendar')
        # XXX When CMF 1.4 is no longer supported, we can instead use:
        # search = catalog.unrestrictedSearchResults(meta_type='CPS Calendar')
        result = []
        for brain in search:
            ob = self.unrestrictedTraverse(brain.getPath(), None)
            if ob is None:
                # Object that was deleted but not unindexed, probably.
                # That shouldn't really happen, so I should have a debug log here.
                continue 
            if not _checkPermission('Invite attendee', ob):
                # You can't invite this guy, so skip him.
                continue
            
            # Filter on attendee_type
            if (attendee_type is not None and 
                getattr(ob, '_attendee_type', None) != attendee_type):
                continue
                       
            if getattr(ob, '_attendee_type',  'INDIVIDUAL') == 'INDIVIDUAL':
                id = getattr(ob, '_attendee_name',  None)
                if id is None:
                    # BBB: This is an old unsupported calendar object created during
                    # development. Ignore it, so as to not create errors.
                    continue
                type = 'INDIVIDUAL'
                attendee = self.getAttendee('!'.join((type, id)))
                title = attendee.title.decode('ISO-8859-15')
                if title.lower().find(query_str.lower()) == -1:
                    # Not a match.
                    continue
                # Match
                result.append(attendee)
            else:
                # Workspace/resource calendars
                title = ob.title
                if title.lower().find(query_str.lower()) == -1:
                    # Not a match.
                    continue
                # Match
                id = '/'.join(ob.getPhysicalPath())
                type = 'CALENDAR'
                attendee = self.getAttendee('!'.join((type, id)))
                result.append(attendee)
        return result
        
    def getAttendeesOfType(self, attendee_type):
        return self.findByName(None, attendee_type)

    def getCurrentUserAttendee(self):
        securitymanager = getSecurityManager()
        id = securitymanager.getUser().getId()
        if not id:
            id = 'Anonymous'
        return self.getAttendee('INDIVIDUAL!' + id)

    def getAttendeeTypes(self):
        return ['INDIVIDUAL', 'WORKSPACE', 'RESOURCE']

    def getMainCalendarForAttendeeId(self, attendee_id):
        # XXX unit test!
        prefix, id = attendee_id.split('!', 1)
        try:
            if prefix == 'INDIVIDUAL':
                cpstool = getToolByName(self, 'portal_cpscalendar')
                return cpstool.getHomeCalendarObject(id)
            return self.unrestrictedTraverse(id)
        except (KeyError, AttributeError):
            return None

    def getAttendeeFromSpec(self, vcaladdress):
        # MAILTO: and CUTYPE is sufficient for CPS, for example, rooms and 
        # resources do not have any reasonable unique mail address.
        # For this reason, CPSCalendar put the type in CUTYPE and the
        # id in X-CID (experimental calendar id). We pick that up here
        # and look up the correct Attendee.

        id = vcaladdress.params.get('X-CID', None)
        if id is None:
            return None
        cutype = vcaladdress.params.get('CUTYPE', 'INDIVIDUAL')
        if cutype == 'INDIVIDUAL':
            prefix = cutype
        else:
            prefix = 'CALENDAR'
        if cutype == 'X-WORKSPACE':
            cutype = 'WORKSPACE'
        attendee = self.getAttendee('%s!%s' % (prefix, id))
        if (attendee._attendee_type == 'UNKNOWN' and
            cutype in self.getAttendeeTypes()):
            # Non-existing or non-upgraded attendees:
            attendee._attendee_type = cutype
        return attendee


class CpsAttendee(Attendee):
    """An attende with notifications"""

    def __init__(self, userid, title, attendee_type, on_invite=None):
        if attendee_type == 'INDIVIDUAL':
            prefix = 'INDIVIDUAL'
        else:
            prefix = 'CALENDAR'
        attendee_id = '!'.join([prefix, userid])
        Attendee.inheritedAttribute('__init__')(
            self, attendee_id, title, attendee_type, on_invite)
        self.id = attendee_id
        self.userid = userid
        self.title = title

    def getUserId(self):
        if self.id.startswith('INDIVIDUAL'):
            return self.userid
        return None

class CpsCalendarTool(UniqueObject, SimpleItem):
    """
    This tool gives CPSSharedCalendar some compatibility with CPSCalendar
    """

    id = 'portal_cpscalendar'
    meta_type = 'CPS Shared Calendar Tool'
    title = 'CPS Shared Calendar Tool'

    security = ClassSecurityInfo()

    create_member_calendar = 1
    member_can_create_home_calendar = 1
    
    security.declarePublic('getHomeCalendarObject')
    def getHomeCalendarObject(self, id=None, verifyPermission=0):
        """Return a member's home calendar object, or None."""
        mtool = getToolByName(self, 'portal_membership')
        if id is None:
            member = mtool.getAuthenticatedMember()
            if not hasattr(member, 'getMemberId'):
                return None

            id = member.getMemberId()

        member_home = mtool.getHomeFolder(id, verifyPermission)
        if member_home:
            try:
                calendar = member_home._getOb('calendar')
                if verifyPermission and not _checkPermission('View', calendar):
                    # Don't return the folder if the user can't get to it.
                    return None
                return calendar
            except AttributeError:
                pass
        return None

    security.declarePublic('getHomeCalendarUrl')
    def getHomeCalendarUrl(self, id=None, verifyPermission=0):
        """Return a member's home calendar url, or None."""
        calendar = self.getHomeCalendarObject(id, verifyPermission)
        if calendar is not None:
            return calendar.absolute_url()
        else:
            return None

    security.declarePublic('createMemberCalendar')
    def createMemberCalendar(self, member_id=None, REQUEST=None):
        """Create a calendar in the home folder of a member

        member_id: member's id for which we create this calendar

        Precondition: member_id must be a valid user id
        """
        user = _getAuthenticatedUser(self)
        user_id = user.getUserName()
        if not member_id:
            member = user
            member_id = user_id
        elif user_id == member_id:
            member = user
        else:
            if _checkPermission(manage_users, self):
                member = self.acl_users.getUserById(member_id, None)
                if member:
                    member = member.__of__(self.acl_users)
                else:
                    raise ValueError("Member %s does not exist" % member_id)
            else:
                raise 'Unauthorized', manage_users
                
        mtool = getToolByName(self, 'portal_membership')
        ttool = getToolByName(self, 'portal_types')

        context = mtool.getHomeFolder(member_id)
        if not context:
            raise ValueError, "Can not create calendar for members without a home folder."

        # Check that the user has the permissions.
        if not _checkPermission(AddPortalContent, context):
            raise 'Unauthorized', AddPortalContent

        membersdir = getToolByName(self, 'portal_directories').members
        entry = membersdir._getEntry(member_id)
        title = entry[membersdir.title_field]
        
        wtool = getToolByName(self, 'portal_workflow')
        wtool.invokeFactoryFor(context, 'CPS Calendar', CALENDAR_ID,
                               title=title, attendee_name=member_id,
                               attendee_type='INDIVIDUAL')
        calendar = context._getOb(CALENDAR_ID)
        
        calendar_type_info = ttool.getTypeInfo('CPS Calendar')
        calendar_type_info._finishConstruction(calendar)

        # Grant ownership to Member (needed if it's created by a Manager, or so)
        calendar.changeOwnership(member)
        calendar.manage_setLocalRoles(member_id, ['Owner', 'WorkspaceManager', 'AttendeeManager'])

        # Rebuild the tree with corrected local roles.
        # This needs a user that can View the object.
        portal_eventservice = getToolByName(self, 'portal_eventservice')
        portal_eventservice.notify('sys_modify_security', calendar, {})

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(
                '%s/%s' % (self.portal_cpscalendar.getHomeCalendarUrl(),
                           CALENDAR_ID))

        return calendar

InitializeClass(CpsCalendarTool)

