# -*- coding: ISO-8859-15 -*-
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
# Author: Lennart Regebro <regebro@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: test_notification.py 27799 2005-10-01 13:42:35Z lregebro $

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest
import email
from datetime import datetime, timedelta

from OFS.SimpleItem import SimpleItem
import transaction

try:
    from Products.CPSSubscriptions.EventManager import get_event_manager
except ImportError:
    # Using CPS < 3.3
    def dummy_event_manager():
        pass
    def get_event_manager():
        return dummy_event_manager

from calcore import cal
from calendartest import CalendarTest, manager_id, user_id

class DummyMailHost(SimpleItem):
    """Host that stores the sent mails in a list
    
    The list can then be inspected. This way you can see who got notified.
    """
    
    mail_log = []
    
    def clearLog(self):
        self.mail_log = []
        
    def send(self, raw_message):
        message = email.message_from_string(raw_message)
        mfrom = message['From']
        mto = message['To']
        subject = message['Subject']
        self.mail_log.append({'from': mfrom, 'to': mto, 'message': message, 'subject': subject})

class TestNotifications(CalendarTest):

    def afterSetUp(self):
        CalendarTest.afterSetUp(self)

        # Set up shortcuts:
        self._as = self.portal.portal_calendar.attendee_source
        self._sm = self.portal.portal_calendar.storage_manager
        self.person1 = self._as.getAttendee('INDIVIDUAL!' + user_id)
        self.person2 = self._as.getAttendee('INDIVIDUAL!' + manager_id)
        
        # Create an event to test on:
        self.event = self.person1.createEvent(
            dtstart=datetime(2005, 2, 10, 14, 00),
            duration=timedelta(minutes=60),
            title='An event')

        # Make sure the users have emails
        mtool = self.portal.portal_directories.members
        entry = mtool.getEntry(user_id)
        entry['email'] = 'test_user_1_@bar.no'
        mtool.editEntry(entry)
        entry = mtool.getEntry(manager_id)
        entry['email'] = 'manager@bar.no'
        mtool.editEntry(entry)

        # Set up the dummy mailhost
        self.portal._delObject('MailHost')
        self.portal._setObject('MailHost', DummyMailHost())
        self._mh = self.portal.MailHost
        # Get rid of any pending notifications
        transaction.commit()
        self._mh.clearLog()
    
    def test_notificationOnInvite(self):
        # An invitaton will send an invitation to the organizer (person1) and
        # the invited attendee (person2). However, it won't notify the person
        # who did the invitation. So, we log in, to make sure we are 
        # the correct user:
        self.login(user_id)
        # And invite somebody else
        self.event.invite([self.person2])
        # Make sure one, and only one invitation was sent
        # With CPS 3.3.something we now need to force the subscription tool
        # to send the mails by committing the transaction.
        em = get_event_manager()
        em()

        self.failIfEqual(len(self._mh.mail_log), 0, 
                         'No notification was sent')
        self.failUnlessEqual(len(self._mh.mail_log), 1, 
                             'More than one notification was sent')
        mail = self._mh.mail_log[0]
        # And that this was sent to the other guy (not the one who is logged in)
        self.failUnlessEqual(mail['to'], 'manager@bar.no', 
                             'Wrong person invited')
        # And check that it was the right event too:
        self.failUnless(mail['subject'].find('invite') != -1 )

    def test_notificationOnAccept(self):
        self._mh.clearLog()
        self.event.setParticipationStatus(self.person2, 'ACCEPTED')
        # Force the evenst to be sent:
        em = get_event_manager()
        em()

        self.failIfEqual(len(self._mh.mail_log), 0, 'No notification was sent')
        self.failUnlessEqual(len(self._mh.mail_log), 1, 
                             'More than one notification was sent')
        mail = self._mh.mail_log[0]
        self.failUnlessEqual(mail['to'], 'test_user_1_@bar.no', 
                             'Wrong person invited')
        self.failUnless(mail['subject'].find('attendence status') != -1 )


    def test_notificationOnEdit(self):
        # There is a functional tests for this as well.
        # We also need to test with a functional test that editing really
        # notifies. The best would be if changing the dtstart and duration 
        # attributes automatically caused a notification, but that is not
        # possible with old style classes, and hence would break Zope 2.7
        # compatibility. Also, it would require us to have a bridge between the
        # Zope3 events and the CPS events, which we currently do not have.
        self.event.location = 'new location' 
        self._sm.notifyOnEventModification(self.event)
        # Commit
        em = get_event_manager()
        em()
        
        self.failIfEqual(len(self._mh.mail_log), 0, 'No notification was sent')
        self.failUnlessEqual(len(self._mh.mail_log), 1, 
                             'More than one notification was sent')
        mail = self._mh.mail_log[0]
        self.failUnlessEqual(mail['to'], 'test_user_1_@bar.no', 
                             'Wrong person invited')
        self.failUnless(mail['subject'].find('modification') != -1 )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(TestNotifications),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
