# -*- coding: ISO-8859-15 -*-
#
# The code for DivDatetimeWidget, DivDateWidget and supporting methds is ZPL. 
# It's based on code by Gary Poster, gary@zope.com and then extended a lot.
# We have been given permission by Gary Poster to include it in this product.
#
# The rest of the code is:
#
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com> 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: widget.py 28241 2005-10-13 11:04:19Z lregebro $

from time import strptime
from datetime import time, timedelta, datetime, date
from zope.app.form.browser import DatetimeWidget, DateWidget, TextWidget
from zope.app.form.browser.widget import renderElement
from zope.app.datetimeutils import DateTimeError
from zope.app.datetimeutils import tzinfo, parseDatetimetz
from zope.app.form.interfaces import ConversionError, WidgetInputError
from zope.app.form.interfaces import InputErrors
from zope.schema.interfaces import ValidationError

# If there is no Localizer, fallback to Zope3 i18n
from zope.i18nmessageid import MessageIDFactory
_ = MessageIDFactory("Default")

# List of the supported languages for the jscalendar.
# These must be synced with what is configured in the configure.zcml.
# The first one is used as default.
supported_languages = ('en', 'de', 'es', 'fr', 'it', 'nl', 'pt', 'ro',)

try:
    from Products.Localizer.Utils import lang_negotiator
    # Use Localizer language negotiation (it's pretty good)
    def getLocalLanguage(context, request):
        # Request is ignored by this method.
        if getattr(context, 'Localizer', None) is not None:
            localizer = context.Localizer
            langs = localizer.get_supported_languages()
            deflang = localizer._default_language
        else:
            langs = supported_languages
            deflang = supported_languages[0]
            
        lang = lang_negotiator(langs)
        if lang is None:
                return deflang
        return lang
except ImportError:
    # No localizer product
    def getLocalLanguage(context, request):
        # This method matches the HTTP_ACCEPT_LANGUAGE to a list of langauges.
        # It does not understand the concept of sublanguages, such as pt_BR.
        # context is ignored by this method.
        if request is None or request.get('HTTP_ACCEPT_LANGUAGE') is None:
            return supported_languages[0]
    
        # Make a list from the request:
        accept = request.get('HTTP_ACCEPT_LANGUAGE')
        langs = []
        for each in accept.split(','):
            each = each.strip()
            each = each.split(';')
    
            # Get the quality
            if len(each) > 1:
                quality = each[1]            # Get the quality
                quality = quality.strip()
                quality = quality[2:]     # Get the number (remove "q=")
                quality = float(quality)  # Change it to float
            else:
                quality = 1.0
                
            # jscalendar currently has no "sublanguages", so we ignore them:
            lang = each[0][:2] 
            langs.append((quality, lang))
            
        # Sort the list on quality:
        langs.sort()
        langs.reverse()
        
        # Go through the list in sorted on quality order, pick the first to match:
        for q, lang in langs:
            if lang in supported_languages:
                return lang
            
        return supported_languages[0]

def setupLanguage(context, request):
    if getattr(request, 'jscalendar_language', None) is not None:
        # Only do this once per request
        return
    lang = getLocalLanguage(context, request)
    request.jscalendar_language = lang
    

class TimedeltaWidget(TextWidget):

    displayWidth = 10

    def _toFormValue(self, value):
        value = super(TimedeltaWidget, self)._toFormValue(value)
        if value:
            duration = (value.days * 86400) + value.seconds
            hours = duration / 3600
            minutes = (duration % 3600) / 60
            value = '%02d:%02d'% (hours, minutes)
        return value

    def _toFieldValue(self, input):
        if input == self._missing:
            return self.context.missing_value
        else:
            try:
                hours, minutes = input.split(':')
                return timedelta(hours=int(hours), minutes=int(minutes))
            except (DateTimeError, ValueError, IndexError), v:
                raise ConversionError("Invalid duration", v)

    def _getDefault(self):
        # Get this from the field instead
        return timedelta(hours=1)

# This should probably be very refactored. Maybe we should use
# sequence widgets, and separate the date and time into separate
# fields?

class DivDatetimeWidget(DatetimeWidget):
    # NOTE: presentation is only good for non-tzinfo-aware datetimes.

    time_format = '%Y-%m-%d'
    displayWidth = 10

    def _unconvert(self, value):
        if value == self.context.missing_value:
            return self._missing
        elif isinstance(value, datetime):
            localizer = getattr(self.context.context, 'Localizer', None)
            if localizer is not None:
                format = localizer.default(self.time_format)
            else:
                format = _(self.time_format)
            return value.strftime(str(format))
        else:
            return str(value)

    def __call__(self):
        setupLanguage(self.context.context, self.request)
        localizer = getattr(self.context.context, 'Localizer', None)
        if localizer is not None:
            mcat = localizer.default
        else:
            mcat = _
        value = self._getFormValue()
        if value:
            if isinstance(value, datetime):
                datestr = value.strftime(str(mcat(self.time_format)))
                hour = value.hour
                minute = value.minute
            else:
                datestr = str(value)
                hour = int(self.request.form.get(self.name + '_hour', 0))
                minute = int(self.request.form.get(self.name + '_minute', 0))
        else:
            datestr = ''
            hour = int(self.request.form.get(self.name + '_hour', 0))
            minute = int(self.request.form.get(self.name + '_minute', 0))
        res = renderElement(self.tag,
                            type=self.type,
                            name=self.name,
                            id=self.name,
                            value=datestr,
                            cssClass=self.cssClass,
                            style=self.style,
                            size=self.displayWidth,
                            extra=self.extra,
                            title=mcat('YYYY-MM-DD'))
        calendar_res = make_calendar_widget(
            self.request,
            "%s_launch_img" % self.name,
            self.name,
            mcat(self.time_format),
            False, title=mcat('Date selector'))
        hour_res = _makeHourDropDown(self.name + '_hour', selected=hour)
        minute_res = _makeMinuteDropDown(self.name + '_minute', selected=minute)
        return "\n".join([res, calendar_res, hour_res,minute_res])

    def hasInput(self):
        for ext in ('', '_hour', '_minute'):
            if not self.name + ext in self.request.form:
                return False
        return True

    def _getFormInput(self):
        return self.request.form[self.name] + ' ' + \
               self.request.form[self.name + '_hour'] + ':' + \
               self.request.form[self.name + '_minute']

    def _toFieldValue(self, input):
        if input == self._missing:
            return self.context.missing_value
        else:
            try:
                localizer = getattr(self.context.context, 'Localizer', None)
                if localizer is not None:
                    format = localizer.default(self.time_format) + ' %H:%M'
                else:
                    format = self.time_format  + ' %H:%M'
                return str2datetime(input, format)
            except (DateTimeError, ValueError, IndexError), v:
                raise ConversionError("Invalid datetime data", v)

class DivDateWidget(DateWidget):

    time_format = '%Y-%m-%d'
    displayWidth = 10

    def _unconvert(self, value):
        if value == self.context.missing_value:
            return self._missing
        elif isinstance(value, date):
            localizer = getattr(self.context.context, 'Localizer', None)
            if localizer is not None:
                format = localizer.default(self.time_format)
            else:
                format = _(self.time_format)
            return value.strftime(str(format))
        else:
            return str(value)

    def __call__(self):
        setupLanguage(self.context.context, self.request)
        localizer = getattr(self.context.context, 'Localizer', None)
        if localizer is not None:
            mcat = localizer.default
        else:
            mcat = _
        value = self._getFormValue()
        if value and isinstance(value, date):
            datestr = value.strftime(str(mcat(self.time_format)))
        else:
            datestr = ''
        res = renderElement(self.tag,
                            type=self.type,
                            name=self.name,
                            id=self.name,
                            value=datestr,
                            cssClass=self.cssClass,
                            style=self.style,
                            size=self.displayWidth,
                            extra=self.extra,
                            title=mcat('YYYY-MM-DD'))
        calendar_res = make_calendar_widget(
            self.request,
            "%s_launch_img" % self.name,
            self.name,
            mcat(self.time_format),
            False, title=mcat('Date selector'))
        return "%s\n%s" % (res, calendar_res)

    def _toFieldValue(self, input):
        if input == self._missing:
            return self.context.missing_value
        else:
            try:
                localizer = getattr(self.context.context, 'Localizer', None)
                if localizer is not None:
                    format = localizer.default(self.time_format)
                else:
                    format = self.time_format
                return str2datetime(input, format)
            except (DateTimeError, ValueError, IndexError), v:
                raise ConversionError("Invalid datetime data", v)


class TimeWidget(TextWidget):

    displayWidth = 20

    def _getFormInput(self):
        return self.request.form[self.name + '_hour'] + ':' + \
               self.request.form[self.name + '_minute']

    def _toFieldValue(self, input):
        if input == self._missing:
            return self.context.missing_value
        else:
            try:
                hours, minutes = input.split(':')
                return time(hour=int(hours), minute=int(minutes))
            except (TimeError, ValueError, IndexError), v:
                raise ConversionError("Invalid time", v)

    def _getDefault(self):
        # Get this from the field instead
        return time(hour=8,minute=0)

    def hasInput(self):
        for ext in ('_hour', '_minute'):
            if not self.name + ext in self.request.form:
                return False
        return True

    def __call__(self):
        value = self._getFormValue()
        hour_res = _makeHourDropDown(self.name + '_hour', selected=value.hour)
        minute_res = _makeMinuteDropDown(self.name + '_minute', selected=value.minute)
        return "\n".join([hour_res,minute_res])


# TODO: This should be refactored to support am/pm stuff.
def _makeHourDropDown(name, begins=0, ends=24, selected=None):
    """Creates a selects drop down for hours.

    The hour box begins at 'begins' and ends at one before 'ends'.
    If the selected hour is before of after the hours displayed, no
    hour will be selected.
    """
    elements = []
    for hour in range(begins, ends-begins):
        text = '%02d' % hour
        if selected == hour:
            element = renderElement('option',
                                    contents=text,
                                    value=text,
                                    selected='selected')
        else:
            element = renderElement('option',
                                    contents=text,
                                    value=text)
        elements.append(element)
    content = '\n'.join(elements)
    return renderElement('select',
                         name=name,
                         id=name,
                         contents=content)

def _makeMinuteDropDown(name, step=5, selected=None):
    """Creates a selects drop down for minutes.

    The minute box starts at 0 and will process with 'step' minutes at
    a time until it's over 60 minutes.
    If the minute selected is not one of the minutes displayed,
    the nearest *preceding* value will be selected.
    """
    elements = []
    for minute in range(0, 60, step):
        text = '%02d' % minute
        if selected >= minute and selected  < minute+step:
            element = renderElement('option',
                                    contents=text,
                                    value=text,
                                    selected='selected')
        else:
            element = renderElement('option',
                                    contents=text,
                                    value=text)
        elements.append(element)
    content = '\n'.join(elements)
    return renderElement('select',
                         name=name,
                         id=name,
                         contents=content)



# That uses a helper function--you'll see a Zope 2-ism you'll need to
# convert as well.  It's separate because I ended up wanting to use
# it for non-schema generated forms as well...

calendar_js_scripts = """\
<style type="text/css">@import url(++resource++calendar-win2k-1.css);</style>
<script type="text/javascript" src="++resource++calendar.js"></script>
<script type="text/javascript" src="++resource++calendar-%s.js"></script>
<script type="text/javascript" src="++resource++calendar-setup.js"></script>"""

calendar_js_template = """%s
<script type="text/javascript">
Calendar.setup({inputField: "%s", ifFormat: "%s", showsTime: %s,
                button: "%s", singleClick: true});
</script>"""

def make_calendar_widget(request,
    image_id, field_id, format, showsTime,
    image_zope_id='++resource++jscalendar.gif',
    style="cursor: pointer;",
    title="Date selector",
    onmouseover=r"this.style.background='black';",
    onmouseout=r"this.style.background=''"):

    image_tag = '<img src="%s" id="%s" style="%s" title="%s" '\
                'onmouseover="%s" onmouseout="%s"/>' % (
        image_zope_id, image_id, style, title, onmouseover, onmouseout)
         
    showsTime = showsTime and "true" or "false"
    result = calendar_js_template % (
        image_tag, field_id, format, showsTime, image_id)
    if not getattr(request, '_jscalendar_scripts_included', 0):
        request._jscalendar_scripts_included = 1
        lang = getattr(request, 'jscalendar_language', 'en')
        result = (calendar_js_scripts % lang)+ result
    return result

def str2datetime(string, format=None):
    if format is None:
        return parseDatetimetz(string).replace(tzinfo=None)
    return datetime(*strptime(string, format)[:6])

# now for something completely different: a widget to display
# List fields with text contents as a textarea.

from zope.app.form.browser import TextAreaWidget
from sets import Set

class SetTextAreaWidget(TextAreaWidget):
    
    def _toFieldValue(self, value):
        value = super(SetTextAreaWidget, self)._toFieldValue(value)
        if value:
            value = value.split('\n')
        return Set(value)
    
    def _toFormValue(self, value):
        if value:
            value = '\n'.join(list(value))
        else:
            value = ''
        return super(SetTextAreaWidget, self)._toFormValue(value)


class EndBeforeStart(ValidationError):
    __doc__ = """End must come after start"""

class EndDateWidget(DivDatetimeWidget):
    """A special widget for end dates

    This widget will raise an error if it's value is lower than the
    value of the start time widget.
    """
    start_widget_id = 'dtstart'
    
    def _toFieldValue(self, input):
        value = DivDatetimeWidget._toFieldValue(self, input)
        try:
            name = 'field.'+self.start_widget_id
            startinput = self.request.form[name] + ' ' + \
                         self.request.form[name + '_hour'] + ':' + \
                         self.request.form[name + '_minute']
            startvalue = DivDatetimeWidget._toFieldValue(self, startinput)
        except InputErrors:
            return value
            
        if startvalue >= value:
            self._error = WidgetInputError(self.context.__name__, self.label,
                                           EndBeforeStart())
            raise self._error
        
        return value
